/*
 * init.c
 */

static char *rcsid = "$Header: /nfs/globus1/src/master/nexus/init.c,v 1.76 1998/02/12 19:11:54 stepanek Exp $";

#define NEXUS_DEFINE_GLOBALS
#include "internal.h"

#ifdef HAVE_TCP_PROTO
extern void *_nx_pr_tcp_info(void);
#define NEXUS_USE_TCP_PROTO	{"protocols", "tcp", _nx_pr_tcp_info},
#else
#define NEXUS_USE_TCP_PROTO
#endif

#ifdef HAVE_UDP_PROTO
extern void *_nx_pr_udp_info(void);
#define NEXUS_USE_UDP_PROTO	{"protocols", "udp", _nx_pr_udp_info},
#else
#define NEXUS_USE_UDP_PROTO
#endif

#ifdef HAVE_MPL_PROTO
extern void *_nx_pr_mpl_info(void);
#define NEXUS_USE_MPL_PROTO	{"protocols", "mpl", _nx_pr_mpl_info},
#else
#define NEXUS_USE_MPL_PROTO
#endif

#ifdef HAVE_MPI_PROTO
extern void *_nx_pr_mpi_info(void);
#define NEXUS_USE_MPI_PROTO	{"protocols", "mpi", _nx_pr_mpi_info},
#else
#define NEXUS_USE_MPI_PROTO
#endif

#ifdef HAVE_INX_PROTO
extern void *_nx_pr_inx_info(void);
#define NEXUS_USE_INX_PROTO	{"protocols", "inx", _nx_pr_inx_info},
#else
#define NEXUS_USE_INX_PROTO
#endif

#ifdef HAVE_SHM_PROTO
extern void *_nx_pr_shm_info(void);
#define NEXUS_USE_SHM_PROTO	{"protocols", "shm", _nx_pr_shm_info},
#else
#define NEXUS_USE_SHM_PROTO
#endif

#ifdef HAVE_TOTEM_PROTO
extern void *_nx_pr_totem_info(void);
#define NEXUS_USE_TOTEM_PROTO	{"protocols", "totem", _nx_pr_totem_info},
#else
#define NEXUS_USE_TOTEM_PROTO
#endif

#ifdef HAVE_XTP_PROTO
extern void *_nx_pr_xtp_info(void);
#define NEXUS_USE_XTP_PROTO	{"protocols", "xtp", _nx_pr_xtp_info},
#else
#define NEXUS_USE_XTP_PROTO
#endif

#if (defined HAVE_QOS) && (defined BUILD_RESOURCE)
extern void *nexus_overseer_qos_info(void);
#define NEXUS_USE_QOS_OVERSEER {"overseers", "qos", nexus_overseer_qos_info},
#else
#define NEXUS_USE_QOS_OVERSEER
#endif


extern void *_nx_rdb_file_info(void);


static nexus_module_list_t default_module_list[] =
{
    NEXUS_USE_SHM_PROTO
    NEXUS_USE_MPL_PROTO
    NEXUS_USE_MPI_PROTO
    NEXUS_USE_INX_PROTO
    NEXUS_USE_TCP_PROTO
    NEXUS_USE_QOS_OVERSEER
    NEXUS_USE_UDP_PROTO
    NEXUS_USE_TOTEM_PROTO
    NEXUS_USE_XTP_PROTO
    {"rdb", "file", _nx_rdb_file_info},
    {NULL, NULL, NULL},
};


static nexus_bool_t		nexusl_initialized = NEXUS_FALSE;

/*
 * nexus_init()
 *
 * Return:
 *   NEXUS_SUCCESS
 *	Successful initialization
 *   NEXUS_ERROR_ALREADY_INITIALIZED
 *	Nexus was already successfully initialized.
 */
int
nexus_init(int *argc,
	   char ***argv,
	   char *args_env_variable,
	   char *package_designator,
	   nexus_module_list_t additional_modules[])
{
    int			rc = NEXUS_SUCCESS;
    int			local_argc;
    char **		local_argv;
    int *		use_argc;
    char ***		use_argv;
    nexus_context_t *	context;
    nexus_module_list_t *module_list;

    if (nexusl_initialized)
    {
	return(NEXUS_ERROR_ALREADY_INITIALIZED);
    }
    else
    {
	nexusl_initialized = NEXUS_TRUE;
    }
    
    if (!argc || !argv)
    {
	/* The user does not want back argc/argv */
	local_argc = 0;
	use_argc = &local_argc;
	use_argv = &local_argv;
    }
    else
    {
	use_argc = argc;
	use_argv = argv;
    }

    if (additional_modules)
    {
	int i, j;

	/* Count the total number of modules */
	j = 1;
	for (i = 0, j = 1;
	     default_module_list[i].family_name != (char *) NULL;
	     i++, j++)
	    ; /* empty body */
	for (i = 0;
	     additional_modules[i].family_name != (char *) NULL;
	     i++, j++)
	    ; /* empty body */

	/* malloc a new module_list to hold all of the modules */
	NexusMalloc(nexus_init(),
		    module_list,
		    nexus_module_list_t *,
		    (sizeof(nexus_module_list_t) * j));

	/* populate the combined module_list */
	for (i = 0, j = 0;
	     default_module_list[i].family_name != (char *) NULL;
	     i++, j++)
	{
	    module_list[j].family_name = default_module_list[i].family_name;
	    module_list[j].module_name = default_module_list[i].module_name;
	    module_list[j].info_func = default_module_list[i].info_func;
	}
	for (i = 0;
	     additional_modules[i].family_name != (char *) NULL;
	     i++, j++)
	{
	    module_list[j].family_name = additional_modules[i].family_name;
	    module_list[j].module_name = additional_modules[i].module_name;
	    module_list[j].info_func = additional_modules[i].info_func;
	}
	module_list[j].family_name = NULL;
	module_list[j].module_name = NULL;
	module_list[j].info_func = NULL;
    }
    else
    {
	module_list = default_module_list;
    }
    
    /*
     * This will bootstrap the thread package if necessary, so that
     * basic calls such as printf, malloc, etc can be called.
     */
    ports0_preinit();
#ifndef BUILD_LITE
    _nx_thread_preinit();
#endif /* BUILD_LITE */

    /*
     * Do some global initializations
     */
    _nx_pausing_for_fatal = NEXUS_FALSE;
    _nx_pause_on_fatal = NEXUS_FALSE;
    _nx_stdout = stdout;
    /*
    _nx_stdout = stderr;
    */

    /*
     * Split and parse the arguments
     */
    _nx_get_args(use_argc,
		 use_argv,
		 args_env_variable,
		 package_designator);

    /*
     * Initialize the fundamental modules.
     * Everything else depends on these being initialized.
     */
    ports0_init(use_argc, use_argv, package_designator);
    _nx_args_init(use_argc, use_argv);
    _nx_md_init(use_argc, use_argv);
#ifndef BUILD_LITE
    _nx_thread_init(use_argc, use_argv);
#endif /* BUILD_LITE */

    /*
     * Initialize context management code,
     * and create an initial context for this process and this thread.
     * Dependencies:
     */
    _nx_context_init(use_argc, use_argv);
    context = _nx_context_alloc();
    _nx_set_context(context);
    _nx_initial_context = context;

    /*
     * Initialize event driver
     * Dependencies:
     *		_nx_thread_init();
     */
    nexus_fd_init(use_argc, use_argv);

    /*
     * Initialize fault tolerance
     * Dependencies:
     */
    _nx_fault_tolerance_init(use_argc, use_argv);

    /*
     * Initialize rdb
     * Dependencies:
     */
    _nx_rdb_init(use_argc, use_argv, module_list);

    /*
     * Initialize _nx_my_hostname
     * Dependencies:
     *		_nx_rdb_init()
     */
    _nx_md_gethostname(_nx_my_hostname, MAXHOSTNAMELEN);


    /*
     * Initialize transform modules
     * Dependencies:
     */
    nexus_transform_init(use_argc, use_argv, module_list);

#ifdef BUILD_RESOURCE
    /*
     * Initialize overseer modules
     * Dependencies:
     */
    nexus_overseer_init(use_argc, use_argv, module_list);
#endif /* BUILD_RESOURCE */

    /*
     * Initialize protocols
     * Dependencies:
     *		nexus_fd_init()
     */
    _nx_proto_init(use_argc, use_argv, module_list);

    /*
     * Initialize attachment
     * Dependencies:
     *		nexus_fd_init()
     */
    _nx_attach_init(use_argc, use_argv);
    
    /*
     * Initialize transform modules
     * Dependencies:
     */
    _nx_commlink_init(use_argc, use_argv);
    
    /*
     * Initialize buffer management
     * Dependencies:
     */
    _nx_buffer_init(use_argc, use_argv);

    
#ifndef BUILD_LITE
    /*
     * Startup a handler thread
     */
    _nx_thread_create_handler_thread();
#endif /* BUILD_LITE */
    
    _nx_args_cleanup(use_argc, use_argv);

  abort:
    
    if (module_list != default_module_list)
    {
	NexusFree(module_list);
    }

    return(rc);
} /* nexus_init() */


/*
 * nexus_shutdown()
 *
 * Return:
 *   NEXUS_SUCCESS
 *	Successful initialization
 *   NEXUS_ERROR_NOT_INITIALIZED
 *	Nexus is not initialized. (Either nexus_init() has not been called,
 *		or nexus_shutdown() has already been called.)
 */
int nexus_shutdown(void)
{
    int rc = NEXUS_SUCCESS;

    if (!nexusl_initialized)
    {
	return(NEXUS_ERROR_NOT_INITIALIZED);
    }
    else
    {
	nexusl_initialized = NEXUS_FALSE;
    }

    nexus_debug_printf(1, ("nexus_shutdown_nonexiting(): Process exiting\n"));

#ifdef BUILD_RESOURCE
    nexus_overseer_shutdown(NEXUS_FALSE);
#endif
    _nx_proto_shutdown(NEXUS_FALSE);
    _nx_rdb_shutdown();
    nexus_fd_shutdown();
#ifndef BUILD_LITE
    _nx_thread_shutdown_handler_thread();
    _nx_thread_shutdown();
#endif /* BUILD_LITE */
    ports0_shutdown();
    _nx_md_shutdown();

    return(rc);
} /* nexus_shutdown() */


/*
 * nexus_abort()
 *
 * Terminate the computation, using whatever means necessary.
 */
void nexus_abort(void)
{
#ifdef BUILD_RESOURCE
    nexus_overseer_abort();
#endif
    _nx_proto_abort();
    _nx_rdb_abort();
#ifndef BUILD_LITE
    _nx_thread_abort(1);
#endif /* BUILD_LITE */
    _nx_md_abort(1);
} /* nexus_abort() */
