/**********************************************************************
$Id: test_utp.c,v 1.3 1997/10/21 02:37:36 freedman Exp $

$Source: /nfs/globus1/src/master/nexus_utp/test/test_utp.c,v $

$Log: test_utp.c,v $
Revision 1.3  1997/10/21 02:37:36  freedman
Changed prefixes of global variables to NEXUS_UTP_.

Revision 1.2  1997/10/20 23:57:03  freedman
Started renaming of functions from UTP_ to nexus_utp_

Revision 1.1  1997/07/23 22:04:12  tuecke
First pass at integrating Dave Kohr UTP timing package into nexus

Revision 1.2  1994/11/03 19:53:02  kohr
Move current source from RCS to CVS control.

Revision 1.1  1994/10/25  20:42:43  tuecke
An initial version of Dave Kohr's timing package

Test functionality of the Unnamed Timing Package (UTP).
**********************************************************************/

/*
Pre-CVS version information:

Revision 1.3  1994/11/03  02:25:17  kohr
Added enable/disable and key/value attributes.

Revision 1.2  1994/10/07  03:54:53  kohr
Test nexus_utp_reset_timer() and UTP_get_accum_time().

Revision 1.1  1994/09/14  06:04:43  kohr
Initial revision
*/

#include <stdio.h>
#include <unistd.h>

#include "../nexus_utp.h"

/*
The tests: create 5 timers, use them like this:

timer 0: use 3 times, accumulate about 5 seconds; disable and stop
	(produces no warning); name it "Timer 0".
timer 1: use once, then reset (looks like it wasn't used at all);
	start/stop it when all are disabled; name it "Timer \"one\"".
timer 2: enable, stop before starting; name it "Timer 2.0".
timer 3: start without stopping
timer 4: disable, start/stop it, enable, start/stop to accumulate about 1
	second
timer 5: try to use it (it is bogus)

Also disable all timers, start/stop timer 1, enable all again.

(All times accumulated using sleep(3).)

Rename output file to "utp_dump_test.txt".

Then nexus_utp_shutdown() and exit().
*/

int main(int argc, char **argv)
{
	double accumTime0;
	int prec;

	if (nexus_utp_init(5, NEXUS_UTP_MODE_SHARED)) {
		fprintf(stderr, "nexus_utp_init() failed, exiting\n");
		return 1;
	}

	nexus_utp_set_attribute("expected-result-%s", "0", "about %d", 5);
	nexus_utp_set_attribute("expected-name-0", "", "Timer 0");
	nexus_utp_set_attribute("expected-count-0", "", "%d", 3);
	nexus_utp_set_attribute("expected-result-%s", "1", "0");
	nexus_utp_set_attribute("expected-name-%s", "1", "Timer \\\"one\\\"");
	nexus_utp_set_attribute("expected-count-1", "", "0");
	nexus_utp_set_attribute("expected-result-%s", "2", "0");
	nexus_utp_set_attribute("expected-name-2", "", "Timer 2.000000");
	nexus_utp_set_attribute("expected-count-2", "", "0");
	nexus_utp_set_attribute("expected-result-%s", "3", "about %.1f", 4.0);
	nexus_utp_set_attribute("expected-name-3", "", "\\\"\\\"");
	nexus_utp_set_attribute("expected-count-3", "", "%.0f", 1.0);
	nexus_utp_set_attribute("expected-result-4", "", "about %.1lf", 1.0);
	nexus_utp_set_attribute("expected-name-4", "", "\\\"\\\"");
	nexus_utp_set_attribute("expected-count-4", "", "1");

	nexus_utp_set_attribute("expected-message-1", "",
			  "nexus_utp_stop_timer: timer 2 not running");
	nexus_utp_set_attribute("expected-message-2", "",
			  "nexus_utp_start_timer: illegal timer 5");
	nexus_utp_set_attribute("expected-message-3", "",
			  "Accumulated 5.015066 seconds in timer 0.");
	nexus_utp_set_attribute("expected-message-4", "",
			  "nexus_utp_shutdown: timer 3 still running");

	nexus_utp_name_timer(0, "Timer 0");
	nexus_utp_name_timer(1, "Timer \\\"%s\\\"", "one");
	nexus_utp_name_timer(2, "Timer %f", 2.0);

	nexus_utp_disable_timer(4);
	nexus_utp_start_timer(4);
	nexus_utp_stop_timer(4);
	nexus_utp_enable_timer(4);

	nexus_utp_start_timer(0);
	nexus_utp_start_timer(4);
	sleep(1);
	nexus_utp_stop_timer(0);
	nexus_utp_stop_timer(4);

	nexus_utp_start_timer(1);
	sleep(1);
	nexus_utp_stop_timer(1);
	nexus_utp_reset_timer(1);
	nexus_utp_disable_all_timers();
	nexus_utp_start_timer(1);
	nexus_utp_stop_timer(1);
	nexus_utp_enable_all_timers();

	nexus_utp_enable_timer(2);
	nexus_utp_stop_timer(2);
	nexus_utp_start_timer(3);
	nexus_utp_start_timer(5);

	nexus_utp_start_timer(0);
	sleep(3);
	nexus_utp_stop_timer(0);

	nexus_utp_start_timer(0);
	sleep(1);
	nexus_utp_stop_timer(0);
	nexus_utp_disable_timer(0);
	nexus_utp_stop_timer(0);

	nexus_utp_get_accum_time(0, &accumTime0, &prec);
	printf("Accumulated %.*lf seconds in timer 0.\n", prec,
	       accumTime0);

	nexus_utp_set_filename("utp_dump_test.txt");

	nexus_utp_shutdown();

	return 0;
}
