/******************************************************************************
CIF_Connection_Nexus.C

Description:

  

CVS Information:

  $Source$
  $Date$
  $Revision$
  $State$
  $Author$
******************************************************************************/

/******************************************************************************
			     Include header files
******************************************************************************/
#include "CIF_Connection_Nexus.H"
#include "Nexus_internal.H"

#include <stdio.h>
#include <string.h>


/******************************************************************************
		       CIF_Connection_Nexus API methods
******************************************************************************/

/*
 * CIF_Connection_Nexus::CIF_Connection_Nexus()
 */
CIF_Connection_Nexus::CIF_Connection_Nexus()
{
    /*
     * Initialize mutex variable
     */
    nexus_mutex_init(&mutex, (nexus_mutexattr_t *) GLOBUS_NULL);

    /*
     * Initialize object state
     */
    object_state = OBJECT_STATE_INACTIVE;
}
/* CIF_Connection_Nexus::CIF_Connection_Nexus() */


/*
 * CIF_Connection_Nexus::open(char *)
 */
CIF_Bool
CIF_Connection_Nexus::open(
    char *				url,
    int					timeout)
{
    Nexus_Monitor			monitor(&mutex);

    if (establish_Connection(url, endpoint, startpoint, timeout))
    {
	object_state = OBJECT_STATE_ACTIVE;
	return true;
    }
    else
    {
	return false;
    }
}
/* CIF_Connection_Nexus::open(char *) */


/*
 * CIF_Connection_Nexus::open(Nexus_Endpoint &)
 */
CIF_Bool
CIF_Connection_Nexus::open(
    Nexus_Endpoint &			local_ctrl_endpoint,
    int					timeout)
{
    Nexus_Monitor			monitor(&mutex);

    if (establish_Connection(local_ctrl_endpoint,
			     endpoint,
			     startpoint,
			     timeout))
    {
	object_state = OBJECT_STATE_ACTIVE;
	return true;
    }
    else
    {
	return false;
    }
}
/* CIF_Connection_Nexus::open(Nexus_Endpoint &) */


/*
 * CIF_Connection_Nexus::send()
 */
CIF_Bool
CIF_Connection_Nexus::send(
    CIF_Byte *				msg_buf,
    CIF_Size				msg_len,
    int					msg_tag)
{
    Nexus_Send_Buffer			buffer;
    Nexus_Size				buffer_size;
    int					msg_len_int = msg_len;

    buffer_size = buffer.sizeof_Int(2);
    buffer_size += buffer.sizeof_Byte((Nexus_Size) msg_len);
    buffer.activate(buffer_size);
    buffer.put_Int(&msg_len_int, 1);
    buffer.put_Int(&msg_tag, 1);
    buffer.put_Byte(msg_buf, msg_len);

    {
	/* XXX: verify state */
    
	Nexus_Monitor			monitor(&mutex);

	if (startpoint.send(buffer))
	{
	    return true;
	}
	else
	{
	    buffer.deactivate();
	    return false;
	}
    }
}
/* CIF_Connection_Nexus::send() */


/*
 * CIF_Connection_Nexus::receive()
 */
CIF_Bool
CIF_Connection_Nexus::receive(
    CIF_Message &			message,	
    int					timeout)
{
    Nexus_Monitor			monitor(&mutex);

    CIF_Message_Info			msg_info;
    int					msg_size;
    int					msg_tag;
    
    Nexus_Receive_Buffer		buffer;

    if (endpoint.receive(buffer, timeout))
    {
	buffer.get_Int(&msg_size, 1);
	buffer.get_Int(&msg_tag, 1);
	
	msg_info.size = (CIF_Size) msg_size;
	msg_info.tag = msg_tag;

    	message.activate(msg_info);
	buffer.get_Byte(message.get_Data(), msg_info.size);

	return true;
    }
    else
    {
	return false;
    }
}
/* CIF_Connection_Nexus::receive() */


/*
 * CIF_Connection_Nexus::poll()
 */
CIF_Bool
CIF_Connection_Nexus::poll()
{
    return endpoint.poll();
}
/* CIF_Connection_Nexus::poll() */


/*
 * CIF_Connection_Nexus::close()
 */
CIF_Bool
CIF_Connection_Nexus::close()
{
    Nexus_Monitor			monitor(&mutex);

    /* XXX: verify state */
    
    object_state = OBJECT_STATE_INACTIVE;
    if (startpoint.deactivate() && endpoint.deactivate())
    {
	return true;
    }
    else
    {
	return false;
    }
}
/* CIF_Connection_Nexus::close() */


/*
 * CIF_Connection_Nexus::~CIF_Connection_Nexus()
 */
CIF_Connection_Nexus::~CIF_Connection_Nexus()
{
    /* XXX: verify state */
    
    nexus_mutex_destroy(&mutex);

    object_state = OBJECT_STATE_DELETED;
    CIF::status = CIF::STATUS_SUCCESS;
}
/* CIF_Connection_Nexus::~CIF_Connection_Nexus() */



/******************************************************************************
		       CIF_Connection_Nexus local methods
******************************************************************************/



