/******************************************************************************
CIF_Connection_Nexus_ROM.C

Description:

  Connection class for Nexus using ROM / reliable ordered multicast

CVS Information:

  $Source$
  $Date$
  $Revision$
  $State$
  $Author$
******************************************************************************/

/******************************************************************************
			     Include header files
******************************************************************************/
#include "CIF_Connection_Nexus_ROM.H"
#include "Nexus_internal.H"

/*
 * CIF_Connection_Nexus_ROM::establish_Connection()
 */
CIF_Bool
CIF_Connection_Nexus_ROM::establish_Connection(
    ARG_IN char *			url,
    ARG_OUT Nexus_Endpoint &		local_comm_endpoint,
    ARG_OUT Nexus_Startpoint &		remote_comm_startpoint,
    ARG_IN int				timeout)
{
    Nexus_Bool				rc;
    nexus_proto_info_totem_t		proto_info;
    globus_url_t			url_info;

    if (globus_url_parse(url, &url_info) != GLOBUS_SUCCESS)
    {
	return false;
    }

    if (strcmp(url_info.scheme, CIF_PROTOCOL_ROM_STR) != 0)
    {
	rc = false;
	goto fn_abort;
    }

    proto_info.process_group = atoi(url_info.url_path);

    
    /*
     * Create endpoint and startpoint
     */
    rc = local_comm_endpoint.activate(NEXUS_PROTO_TYPE_TOTEM,
				      &proto_info,
				      sizeof(proto_info));
    if (rc == false)
    {
	goto fn_abort;
    }

    rc = remote_comm_startpoint.activate(local_comm_endpoint);
    if (rc == false)
    {
	local_comm_endpoint.deactivate();
	goto fn_abort;
    }

    rc = true;
    
  fn_abort:
    globus_url_destroy(&url_info);
    
    return rc;
}
/* CIF_Connection_Nexus::establish_Connection() */


/*
 * CIF_Connection_Nexus_ROM::establish_Connection()
 */
CIF_Bool
CIF_Connection_Nexus_ROM::establish_Connection(
    ARG_IN Nexus_Endpoint &		local_ctrl_endpoint,
    ARG_OUT Nexus_Endpoint &		local_comm_endpoint,
    ARG_OUT Nexus_Startpoint &		remote_comm_startpoint,
    ARG_IN int				timeout)
{
    return false;
}
/* CIF_Connection_Nexus::establish_Connection() */
