/******************************************************************************
Description:

  Connection class for Nexus using TCP / reliable ordered unicast

CVS Information:

  $Source$
  $Date$
  $Revision$
  $State$
  $Author$
******************************************************************************/

/******************************************************************************
			     Include header files
******************************************************************************/
#include "CIF_Connection_Nexus_TCP.H"
#include "Nexus_internal.H"

/*
 * CIF_Connection_Nexus_TCP::establish_Connection()
 */
CIF_Bool
CIF_Connection_Nexus_TCP::establish_Connection(
    ARG_IN char *			url,
    ARG_OUT Nexus_Endpoint &		local_comm_endpoint,
    ARG_OUT Nexus_Startpoint &		remote_comm_startpoint,
    ARG_IN int				timeout)
{
    Nexus_Timer				timer(timeout);
    
    globus_url_t			url_info;

    if (globus_url_parse(url, &url_info) != GLOBUS_SUCCESS)
    {
	return false;
    }

    if (strcmp(url_info.scheme, CIF_PROTOCOL_TCP_STR) != 0)
    {
	return false;
    }

    /*
     * Establish a connection with the listener
     */
    Nexus_Startpoint			remote_ctrl_startpoint;

    if (remote_ctrl_startpoint.activate(url_info.host, url_info.port) == false)
    {
	return false;
    }
    
    /*
     * Send a startpoint bound to our endpoint
     */
    Nexus_Startpoint			local_comm_startpoint;
    Nexus_Send_Buffer			send_buffer;
    Nexus_Bool				rc;
    
    local_comm_endpoint.activate();
    local_comm_startpoint.activate(local_comm_endpoint);
    send_buffer.activate(
	send_buffer.sizeof_Startpoint(&local_comm_startpoint, 1));
    send_buffer.put_Startpoint(&local_comm_startpoint, 1);
    rc = remote_ctrl_startpoint.send(send_buffer);
    remote_ctrl_startpoint.deactivate();
    if (rc == false)
    {
	local_comm_endpoint.deactivate();
	return false;
    }

    /*
     * Wait for a startpoint from the remote process
     */
    Nexus_Receive_Buffer		recv_buffer;

    if (local_comm_endpoint.receive(recv_buffer, timer.get_Remaining_Time()))
    {
	recv_buffer.get_Startpoint(&remote_comm_startpoint, 1);

	return true;
    }
    else
    {
	local_comm_endpoint.deactivate();
	return false;
    }
}
/* CIF_Connection_Nexus::establish_Connection() */


/*
 * CIF_Connection_Nexus_TCP::establish_Connection()
 */
CIF_Bool
CIF_Connection_Nexus_TCP::establish_Connection(
    ARG_IN Nexus_Endpoint &		local_ctrl_endpoint,
    ARG_OUT Nexus_Endpoint &		local_comm_endpoint,
    ARG_OUT Nexus_Startpoint &		remote_comm_startpoint,
    ARG_IN int				timeout)
{
    /*
     * Wait for a startpoint from the remote process
     */
    Nexus_Receive_Buffer		recv_buffer;

    if (local_ctrl_endpoint.receive(recv_buffer, timeout) == false)
    {
	return false;
    }
    recv_buffer.get_Startpoint(&remote_comm_startpoint, 1);

    /*
     * Send a startpoint bound to our endpoint
     */
    Nexus_Startpoint			local_comm_startpoint;
    Nexus_Send_Buffer			send_buffer;
    
    local_comm_endpoint.activate();
    local_comm_startpoint.activate(local_comm_endpoint);
    send_buffer.activate(
	send_buffer.sizeof_Startpoint(&local_comm_startpoint, 1));
    send_buffer.put_Startpoint(&local_comm_startpoint, 1);
    if (remote_comm_startpoint.send(send_buffer) == false)
    {
	local_comm_endpoint.deactivate();
	remote_comm_startpoint.deactivate();
	return false;
    }

    return true;
}
/* CIF_Connection_Nexus::establish_Connection() */
